HC = require 'hardoncollider'

function love.load()

	screen_width = 1024
	screen_heigth = 800

	win = love.graphics.setMode(screen_width,screen_heigth)
	
	text = {}
	
	---- Collision library ----
	HC.init(100, on_collide)
	
	spaceship_sprite = love.graphics.newImage("enemy.png")

	spaceship = {	
		x = 512,
		y = 400,
		speed = 200,
		rotation_speed = 6,
		angle = 0,
		hc_shape = HC.addRectangle( 512 - spaceship_sprite:getWidth()/2,
									400 - spaceship_sprite:getHeight()/2,
									spaceship_sprite:getWidth(), spaceship_sprite:getHeight() ),
		hp = 50
	}
	
	HC.addToGroup( "player_team", spaceship.hc_shape )
	
	screen_box = HC.addRectangle( 0, 0, screen_width, screen_heigth)
	
	dx = 0
	dy = 0
	
	spaceship.hc_shape.parent = spaceship
   
	gun = {
		x = spaceship.x + spaceship_sprite:getWidth()/2,
		y = spaceship.y
	};
	
	game_level = 1
	
	---- Manage bullets ----
	ammos = {};
	ammo_sprite = love.graphics.newImage("ammo.png")
	ammo_limit = 200
	ammo_timer = 0
	ammo_interval = 0.1
	ammo_default_speed = 500
	weapon_level = 4
	
	---- Manage stars ----
	stars = {};
	star_sprite = love.graphics.newImage("star.png")
	star_limit = 100
	star_timer = 0
	star_interval = 0.5
	star_default_speed = 400
		
	-- Throws in some stars to populate the screen before starting the game
	for i=1, 70 do
		local star = {}
		star.x = math.random(0,screen_width)
		star.y = math.random(0,screen_heigth)
		star.size_factor = math.random(20,250)/400
		star.speed = star_default_speed
		table.insert(stars,star)
	end
	
	---- Manage enemies ----
	enemies = {};
	enemy_sprite = love.graphics.newImage("enemy2.png")
	enemy_limit = 100
	enemy_timer = 0
	enemy_interval = 0.5
	enemy_default_speed = 200
	
end

function love.update(dt)
	-- check for collisions
    HC.update(dt)
	
	if (spaceship.hp <= 0) then						
		restart()		
	end
	
	get_keys(dt)		
	update_spaceship(dt)
	update_stars(dt)
	update_ammo(dt)	
	update_enemies(dt)
end

function love.draw()
	
	for i = 1, #ammos do
		if ammos[i] ~= nil then
			love.graphics.draw( ammo_sprite, ammos[i].x, ammos[i].y, ammos[i].angle, 1, 1, 
			ammo_sprite:getWidth() / 2, ammo_sprite:getHeight() / 2 )			
		end
	end
	
	for i=1, #stars do
		if stars[i] ~= nil then
			love.graphics.draw( star_sprite, stars[i].x, stars[i].y, 0, stars[i].size_factor)
		end
	end
	
	for i=1, #enemies do
		if enemies[i] ~= nil then						
			love.graphics.draw( enemy_sprite, enemies[i].x, enemies[i].y, 0, enemies[i].size_factor,
								enemies[i].size_factor, enemy_sprite:getWidth() / 2, enemy_sprite:getHeight() / 2)						
								
		end
	end
	
	love.graphics.draw( spaceship_sprite, spaceship.x, spaceship.y, spaceship.angle,
						1, 1, spaceship_sprite:getWidth() / 2, spaceship_sprite:getHeight() / 2 )			
	
end

function love.keypressed(key)
	if key == "q" or key == "escape" then
		love.event.push('q')
	end
	
	if key == "u" then
		weapon_level = weapon_level + 1
	end
	
	if key == "p" then
		weapon_level = weapon_level - 1
	end
end

function restart()
	for k, enemy in pairs (enemies) do		
		enemy.remove = true
	end
	
	for k, ammo in pairs (ammos) do		
		ammo.remove = true
	end
	
	spaceship.x = 512
	spaceship.y = 700
	spaceship.hp = 50
	
	start_timer = 0
end

function get_keys(dt)	
	
	dx = dx * 0.95
	dy = dy * 0.95
	
	if love.keyboard.isDown( "right" ) 
	   or love.keyboard.isDown( "d" )  then
		dx = spaceship.speed
	elseif love.keyboard.isDown( "left" )
		   or love.keyboard.isDown( "a" )	then
		dx = - spaceship.speed
	end

	if love.keyboard.isDown( "down" )
	   or love.keyboard.isDown( "s" ) then
		dy = spaceship.speed
	elseif love.keyboard.isDown( "up" )
		   or love.keyboard.isDown( "w" ) then
		dy = - spaceship.speed
	end
		
	spaceship.x = spaceship.x + dx*dt
	spaceship.y = spaceship.y + dy*dt
	
	spaceship.hc_shape:moveTo( spaceship.x, spaceship.y )
	
end

function update_stars(dt)

	for k, star in ipairs(stars) do
		if star.y > screen_heigth then
			table.remove(stars,k)
		else
			star.y = star.y +( star.speed*dt*( star.size_factor/2) )
		end
	end
	
	if star_timer >= star_interval
	   and #stars < star_limit then
		for i=1, math.random(5,10) do
			local star = {}
			star.x = math.random(0,screen_width)
			star.y = math.random(100,200)*-1
			star.size_factor = math.random(20,250)/400
			star.speed = star_default_speed
			table.insert(stars,star)
		end
		star_timer = 0
	else
		star_timer = star_timer+dt
	end
	
	if #stars > star_limit then
		table.remove(stars,1)
	end
end

function update_ammo(dt)
	if ( ammo_timer < ammo_interval ) then
		ammo_timer = ammo_timer + dt
	end
	
	if #ammos > ammo_limit then		
		HC.removeFromGroup( "player_team", ammos[1].hc_shape )		
		HC.remove( ammos[1].hc_shape )
		table.remove(ammos,1)
	end
	
	for k, ammo in ipairs(ammos) do		
		if ( ammo.x > 1100
			 or ammo.x < -100
			 or ammo.y > 900
			 or ammo.y < -100
			 or ammo.remove == true ) then
			HC.removeFromGroup( "player_team", ammo.hc_shape )
			HC.remove( ammo.hc_shape )
			table.remove(ammos,k)
		else			 
			ammo.x = ammo.x + ammo.speed * math.cos( ammo.angle ) * dt
			ammo.y = ammo.y + ammo.speed * math.sin( ammo.angle ) * dt
			ammo.hc_shape:setRotation( ammo.angle )
			ammo.hc_shape:moveTo (ammo.x, ammo.y)
		end
	end
	
	if love.mouse.isDown( "l" ) then
		if ( ammo_timer >= ammo_interval ) then
			 ammo_timer = 0
			for i = 0, weapon_level-1 do
				local ammo = {}
				ammo.x = gun.x
				ammo.y = gun.y
				ammo.angle = spaceship.angle
				if weapon_level ~= 1 then
					ammo.angle = ammo.angle + ( i*30/(weapon_level-1) - 15 )*math.pi/180
				end
				ammo.hc_shape = HC.addRectangle( ammo.x - ammo_sprite:getWidth()/2,
												 ammo.y - ammo_sprite:getHeight()/2,
												 ammo_sprite:getWidth(), 
												 ammo_sprite:getHeight() )
				ammo.hc_shape.sprite = ammo_sprite
				ammo.hc_shape.parent = ammo
				ammo.speed = ammo_default_speed
				ammo.remove = false
				HC.addToGroup( "player_team", ammo.hc_shape )
				table.insert(ammos,ammo)				
			end
		end
	end
end

function update_spaceship(dt)

	local mouse = {}
	
	mouse.x, mouse.y = love.mouse.getPosition()
		
	if math.atan2( mouse.y - gun.y, mouse.x - gun.x) < spaceship.angle
	   and math.atan2( mouse.y - gun.y, mouse.x - gun.x) > spaceship.angle-math.pi then
	   
		spaceship.angle = spaceship.angle + - spaceship.rotation_speed*dt
		
		if spaceship.angle < -math.pi then
			spaceship.angle = math.pi
		end
	end

	if math.atan2( mouse.y - gun.y, mouse.x - gun.x) > spaceship.angle 
	   and math.atan2( mouse.y - gun.y, mouse.x - gun.x) < spaceship.angle+math.pi then
		
		spaceship.angle = spaceship.angle + spaceship.rotation_speed*dt
	   
		if spaceship.angle > math.pi then
			spaceship.angle = -math.pi					
		end
	end

	if math.atan2( mouse.y - gun.y, mouse.x - gun.x) > spaceship.angle+math.pi then
		spaceship.angle = spaceship.angle - spaceship.rotation_speed*dt
		
		if spaceship.angle < -math.pi then
			spaceship.angle = math.pi
		end
	end

	if math.atan2( mouse.y - gun.y, mouse.x - gun.x) < spaceship.angle-math.pi then
		
		spaceship.angle = spaceship.angle + spaceship.rotation_speed*dt

		if spaceship.angle > math.pi then
			spaceship.angle = -math.pi
		end
	end
	
	spaceship.hc_shape:setRotation( spaceship.angle )
	
	gun.x = spaceship.x + spaceship_sprite:getWidth()/2 * math.cos( spaceship.angle )
	gun.y = spaceship.y + spaceship_sprite:getWidth()/2 * math.sin( spaceship.angle )
end

start_timer = 0

function update_enemies(dt)

	if enemy_timer >= enemy_interval
	   and #enemies < enemy_limit
	   and start_timer > 3 then
		for i=1, math.random(0,4) do
			local enemy = {}
			enemy.x = math.random(0,1024)
			enemy.y = math.random(100,200)*-1
			enemy.size_factor = math.random(100,250)/200
			enemy.speed = enemy_default_speed
			enemy.hc_shape = HC.addCircle( enemy.x,
										   enemy.y,
										   enemy.size_factor*enemy_sprite:getWidth()/2 )
			enemy.hc_shape.parent = enemy
			enemy.hc_shape.sprite = enemy_sprite
			enemy.remove = false
			HC.addToGroup( "enemy_team", enemy.hc_shape )
			table.insert(enemies,enemy)
		end
		enemy_timer = 0
		start_timer = 3
	else
		start_timer = start_timer+dt
		enemy_timer = enemy_timer+dt
	end
	
	for k, enemy in ipairs(enemies) do
		if ( enemy.y > 900
			 or enemy.remove == true ) then				
			HC.removeFromGroup( "enemy_team", enemy.hc_shape )
			HC.remove( enemy.hc_shape )				
			table.remove(enemies,k)
		else
			enemy.y = enemy.y+( enemy.speed*dt/( enemy.size_factor ) )
			enemy.hc_shape:moveTo( enemy.x, enemy.y )			
		end
	end	
end

function on_collide(dt, shape_a, shape_b)	
		
	if ( shape_a == spaceship.hc_shape 
		 and shape_b.sprite == enemy_sprite ) then 
		HC.setGhost( shape_b )
		shape_b.parent.remove = true
		spaceship.hp = spaceship.hp-10
		return
	end
	
	if ( shape_b == spaceship.hc_shape 
	     and shape_a.sprite == enemy_sprite ) then 
		HC.setGhost( shape_a )	
		shape_a.parent.remove = true
		spaceship.hp = spaceship.hp-10		
		return
	end
	
	if ( shape_a.sprite == enemy_sprite 
		 and shape_b.sprite == ammo_sprite  ) then
		shape_a.parent.speed = shape_a.parent.speed + 50
		if shape_a.parent.speed > 700 then
			HC.setGhost( shape_a )			
			shape_a.parent.remove = true
		end
		HC.setGhost( shape_b )
		shape_b.parent.remove = true
		return
	end
	
	if ( shape_b.sprite == enemy_sprite 
		 and shape_a.sprite == ammo_sprite ) then
		shape_b.parent.speed = shape_b.parent.speed + 50		
		if shape_b.parent.speed > 700 then
			HC.setGhost( shape_b )			
			shape_b.parent.remove = true		
		end
		HC.setGhost( shape_a )	
		shape_a.parent.remove = true		
		return
	end
end